/*
 * eID Chrome project.
 *
 * Copyright 2017-2021 e-Contract.be BV. All rights reserved.
 * e-Contract.be BV proprietary/confidential. Use is subject to license terms.
 */

"use strict";

var options;

function pageMessageEvent(event) {
    if (event.source !== window) {
        return;
    }

    if (event.data.type === "eid-chrome-offline-extension-read-identity") {
        options = event.data.options;
        var startMessage = {
            type: "eid-chrome-extension-start"
        };
        window.postMessage(startMessage, "*");
        return;
    }

    if (typeof options === "undefined") {
        // do not interfere with online eID Chrome Extension activities
        return;
    }

    if (event.data.type === "eid-chrome-extension-response") {
        var domParser = new DOMParser();
        var xmlDoc = domParser.parseFromString(event.data.data, "text/xml");
        var documentElement = xmlDoc.documentElement;
        if (documentElement.localName === "VersionResponse") {
            var includeAddress = false;
            if (typeof options.includeAddress !== "undefined") {
                includeAddress = options.includeAddress;
            }
            var identificationRequest = "<IdentificationRequest xmlns=\"urn:be:e-contract:eid:protocol:1.0.0\" IncludeAddress=\"" +
                    includeAddress + "\"/>";
            var requestMessage = {
                type: "eid-chrome-extension-request",
                data: identificationRequest
            };
            window.postMessage(requestMessage, "*");
            return;
        }
        if (documentElement.localName === "IdentificationResponse") {

            // we directly terminate the middleware here
            var requestMessage = {
                type: "eid-chrome-extension-request",
                data: "<FinishRequest xmlns=\"urn:be:e-contract:eid:protocol:1.0.0\"/>"
            };
            window.postMessage(requestMessage, "*");

            var identityFileElement = documentElement.getElementsByTagNameNS("urn:be:e-contract:eid:protocol:1.0.0", "IdentityFile")[0];
            var encodedIdentityFile = identityFileElement.textContent;

            var identitySignatureFileElement = documentElement.getElementsByTagNameNS("urn:be:e-contract:eid:protocol:1.0.0", "IdentitySignatureFile")[0];
            var encodedIdentitySignatureFile = identitySignatureFileElement.textContent;

            var encodedAddressFile;
            var encodedAddressSignatureFile;
            var addressFileElements = documentElement.getElementsByTagNameNS("urn:be:e-contract:eid:protocol:1.0.0", "AddressFile");
            if (addressFileElements.length > 0) {
                encodedAddressFile = addressFileElements[0].textContent;
                var encodedAddressSignatureFileElement = documentElement.getElementsByTagNameNS("urn:be:e-contract:eid:protocol:1.0.0", "AddressSignatureFile")[0];
                encodedAddressSignatureFile = encodedAddressSignatureFileElement.textContent;
            }

            var encodedCertificates = [];
            var certificateElements = documentElement.getElementsByTagNameNS("urn:be:e-contract:eid:protocol:1.0.0", "Certificate");
            for (var idx = 0; idx < certificateElements.length; idx++) {
                var certificateElement = certificateElements[idx];
                var encodedCertificate = certificateElement.textContent;
                encodedCertificates[idx] = encodedCertificate;
            }

            var validateIdentityMessage = {
                type: "validate-identity",
                encodedIdentityFile: encodedIdentityFile,
                encodedIdentitySignatureFile: encodedIdentitySignatureFile,
                encodedCertificates: encodedCertificates
            };
            if (typeof encodedAddressFile !== "undefined") {
                validateIdentityMessage.encodedAddressFile = encodedAddressFile;
                validateIdentityMessage.encodedAddressSignatureFile = encodedAddressSignatureFile;
            }
            chrome.runtime.sendMessage(null, validateIdentityMessage, function (responseMessage) {
                // use background (where all the javascript libraries live) to validate
                if (typeof responseMessage === "undefined") {
                    console.log("last error: " + chrome.runtime.lastError);
                    console.log(chrome.runtime.lastError);
                    return;
                }
                if (responseMessage.status === "OK") {
                    var identityMessage = {
                        type: "eid-chrome-offline-extension-identity",
                        data: {
                            identity: responseMessage.identity
                        }
                    };
                    if (typeof responseMessage.address !== "undefined") {
                        identityMessage.data.address = responseMessage.address;
                    }
                    window.postMessage(identityMessage, "*");
                    return;
                }
                var errorMessage = {
                    type: "eid-chrome-offline-extension-identity-error",
                    data: responseMessage.status
                };
                window.postMessage(errorMessage, "*");
            });

            return;
        }

        if (documentElement.localName === "ErrorResponse") {
            var errorMessage = {
                type: "eid-chrome-offline-extension-identity-error",
                data: "EID_CHROME_MIDDLEWARE_ERROR"
            };
            window.postMessage(errorMessage, "*");
            return;
        }

        console.log(documentElement);
    }
}

function init() {
    window.addEventListener("message", pageMessageEvent, false);

    var isInstalledNode = document.createElement('div');
    isInstalledNode.id = 'eid-chrome-offline-extension-is-installed';
    isInstalledNode.setAttribute("style", "display: none;");
    document.body.appendChild(isInstalledNode);
}

init();
